/* ccgo: igs/control.hh
 * 
 * Copyright 2002 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef IGS_CONTROL_HH
#define IGS_CONTROL_HH
#include "conn.hh"
#include "parser/parser.hh"
#include "player.hh"
#include "game.hh"
#include "result.hh"
#include "../move.hh"
#include "storage.hh"
#include "odds.hh"
#include "match.hh"
#include "board.hh"
#include <queue>
#include <vector>
namespace igs
{
	enum PromptType {
		PROMPT_NORMAL = 0,
		PROMPT_MATCH,
		PROMPT_SCORE,
		PROMPT_OBSERVE
	};

	enum ToggleType {
		TOGGLE_BELL,
		TOGGLE_QUIET,
		TOGGLE_SHOUT,
		TOGGLE_AUTOMAIL,
		TOGGLE_OPEN,
		TOGGLE_LOOKING,
		TOGGLE_KIBITZ,
		TOGGLE_CHATTER,
		TOGGLE_SINGLEGAME,
		// the following should be fixed
		TOGGLE_VERBOSE,
		TOGGLE_CLIENT
	};

	class Control :
		virtual public Conn
	{
		Parser * parser;
		void process_line();
		int num_cmd;
		std::string cmd_dat;
		std::queue<std::string> cmd_queue;
	protected:
		void queue_cmd(const std::string &);
		void queue_process();
	public:
		Control();
		virtual ~Control();
		const std::string & get_line();
		bool open(const std::string &);
		void close();
		int get_num_cmd();
		const std::string & get_cmd_dat();
		virtual void client_ready();
		virtual void set_prompt(PromptType);
		virtual void set_username(const std::string &);
		virtual void do_beep();
		virtual void add_message(const std::string &);
		virtual void add_tagged_message(const std::string &, const std::string &);
		virtual void player_update(const Player &);
		virtual void player_stats(const Player &);
		virtual void player_odds(const Odds &);
		virtual void player_end_update();
		virtual void game_result(const std::vector<Result> &);
		virtual void stored_game(const std::vector<Storage> &);
		virtual void game_update(const Game &);
		virtual void game_end_update();
		virtual void game_move(go::Move *, int = 0);
		virtual void game_undo(int = 0);
		virtual void game_time(int, int, int, int, int);
		virtual void ob_remove(int);
		virtual void game_finish(int, const std::string &);

		virtual void match_score(int = 0);
		virtual void match_restore(int = 0);
		virtual void match_remove(const go::Loc &, int = 0);
		virtual void match_adjourn(int = 0);
		virtual void match_finish(const std::string &, int = 0);

		virtual void match_resign(const std::string &);
		virtual void match_timeup(const std::string &);
		virtual void remove_match(const std::string &, const std::string &);

		virtual void player_update(const std::string &, const std::string &);
		virtual void player_remove(const std::string &);

		virtual void message_sys(const std::string &);
		virtual void message_tell(const std::string &);
		virtual void message_tell(const std::string &, const std::string &);
		virtual void message_yell(const std::string &);
		virtual void message_kibitz(int, const std::string &);
		virtual void message_misc(const std::string &);

		virtual void match_request(const Match &);
		virtual void match_create(int, const std::string &);
		virtual void match_decline(const std::string &);
		virtual void game_board(const Board &);

		virtual void toggle_state(ToggleType, bool);
	};
}
#endif // IGS_CONTROL_HH
