/* ccgo: igs/parser/misc.cc
 * 
 * Copyright 2002 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include "misc.hh"
#include <sstream>
#include <iostream>
#include <regexx.hh>

#define atms(i, j) (rx.match[i].atom[j].str())

using namespace igs;
using namespace parser;

extern Misc parser_misc;
Misc parser_misc(& parser_misc);

Parser * Misc::id(Control * c) const
{
	regexx::Regexx rx;
	int cmd = c->get_num_cmd();
	std::string dat = c->get_cmd_dat();
	switch (cmd) {
	case 0: // no cmd??
		if (c->get_line() == "#> ") { // oh oh! not in client mode...
			c->toggle_state(TOGGLE_CLIENT, false);
		} else return 0;
		break;
	case 5: // messages...
		c->message_misc(dat);
		break;
	case 9: // ...
		if (rx.exec(dat, "^Removing game (\\d+) from ob")) {
			int g = -1;
			std::istringstream(atms(0, 0)) >> g;
			if (g < 0) return 0;
			c->ob_remove(g);
		} else if (rx.exec(dat, "\\{Game (\\d+): \\S+ vs \\S+ (.*)\\}")) {
			int g = -1;
			std::istringstream(atms(0, 0)) >> g;
			if (g < 0) return 0;
			std::string d = atms(0, 1);
			if (rx.exec(d, "has adjourned") ) c->game_finish(g, "adjourned");
			else if (rx.exec(d, ": (.*)")) c->game_finish(g, atms(0, 0));
			else {
				std::cerr << "strange game result: " << dat << std::endl;
			}
		} else if (rx.exec(dat, "^Adding game to observation list.")) {
		} else if (rx.exec(dat, "^Set (\\S+) to be (\\S+)\\.")) {
			ToggleType type;
			std::string s = atms(0, 0);

			if (s == "bell") type = TOGGLE_BELL;
			else if (s == "quiet") type = TOGGLE_QUIET;
			else if (s == "shout") type = TOGGLE_SHOUT;
			else if (s == "automail") type = TOGGLE_AUTOMAIL;
			else if (s == "open") type = TOGGLE_OPEN;
			else if (s == "looking") type = TOGGLE_LOOKING;
			else if (s == "kibitz") type = TOGGLE_KIBITZ;
			else if (s == "chatter") type = TOGGLE_CHATTER;
			else if (s == "singlegame") type = TOGGLE_SINGLEGAME;
			else if (s == "verbose") type = TOGGLE_VERBOSE;
			else if (s == "client") type = TOGGLE_CLIENT;
			else return 0; // how can this be?

			bool state = atms(0, 1) == "True";
			c->toggle_state(type, state);
			// std::cerr << "toggling " << atms(0, 0) << " to " << atms(0, 1) << std::endl;
		} else if (rx.exec(dat, "\\!\\!(.*)\\!\\!\\: (.*)")) {
			c->message_sys(dat);
			// std::cerr << "[system message]" << dat << std::endl;
		} else if (rx.exec(dat, "^Game (\\d+): \\S+ vs \\S+ (.*)")) {
			int g = -1;
			std::istringstream(atms(0, 0)) >> g;
			if (g < 0) return 0;
			std::string d = atms(0, 1);
			if (rx.exec(d, "has adjourned") ) c->game_finish(g, "adjourned");
			else if (rx.exec(d, ": (.*)")) c->game_finish(g, atms(0, 0));
			else {
				std::cerr << "strange game result: " << dat << std::endl;
			}
		} else if (rx.exec(dat, "You can check .* finished.")) {
			c->match_score();
		} else if (rx.exec(dat, "Board is restored to .*")) {
// 			std::cerr << "board restored" << std::endl;
			c->match_restore();
		} else if (rx.exec(dat, "Game has been adjourned.")) {
			c->match_adjourn();
		} else if (rx.exec(dat, "Removing @ (([A-H]|[J-T])\\d+)")) {
			c->match_remove(go::Loc(atms(0, 0)));
		} else if (rx.exec(dat, "Requesting match in \\d+ min with \\S+ as")) {
			// match request sent
		} else if (rx.exec(dat, "(\\S+) has resigned the game.")) {
			c->match_resign(atms(0, 0));
		} else if (rx.exec(dat, "(\\S+) has run out of time.")) {
			c->match_timeup(atms(0, 0));
		} else if (rx.exec(dat, "Removed game file (\\S+)-(\\S+) from database.")) {
			c->remove_match(atms(0, 0), atms(0, 1));
		} else if (rx.exec(dat, "Setting your \\S{3} to \\S+")) {
			// setting who to tell
		} else if (rx.exec(dat, "Game saved.")) {
			// adjourned match
		} else if (rx.exec(dat, "game completed.")) {
			// adjourned match
		} else if (rx.exec(dat, "(\\S+) has restarted your game.")) {
			// restarting game
		} else if (rx.exec(dat, "(\\S+) has restored your old game.")) { // what's the difference from the above??
			// restored game
		} else if (rx.exec(dat, "(\\S+) has typed done.")) {
			// done in scoring
		} else if (rx.exec(dat, "The player (\\S+) is now in byo-yomi.")) {
			// enter byo-yomi
		} else if (rx.exec(dat, "You have (\\d+) stones and (\\d+) minute")) {
			// byo-yomi info
		} else if (rx.exec(dat, "Your opponent has lost his/her connection.")) {
			// opponent gone
		} else {
			return 0;
		}
		break;
	case 19: // say
		if (rx.exec(dat, "\\*(\\S+)\\*: (.*)")) {
			c->message_kibitz(0, dat);
		} else return 0;
		break;
	case 20: // match outcome, following a board
		if (rx.exec(dat, "(\\S+) \\(W:O\\): +(\\S+) +to +(\\S+) +\\(B:#\\): +(\\S+)")) {
			std::ostringstream ss;
			ss <<  atms(0, 0) << ' ' << atms(0, 1) << ' ' << atms(0, 2) << ' ' << atms(0, 3);
			c->match_finish(ss.str());
		} else return 0;
		break;
	case 21: // misc
		if (rx.exec(dat, "\\{Match (\\d+): (\\S+) \\[.+\\] vs. (\\S+)")) { // new game
			int g = -1;
			std::istringstream(atms(0, 0)) >> g;
			if (g < 0) return 0;
			Game gm;
			gm.set_num(g);
			gm.set_white(atms(0, 1));
			gm.set_black(atms(0, 2));
			gm.set_moves(0);
			c->game_update(gm);			
		} else if (rx.exec(dat, "\\{Game (\\d+): (\\S+) vs (\\S+) @ Move (\\d+)")) {  // resumed game
			int g = -1;
			std::istringstream(atms(0, 0)) >> g;
			if (g < 0) return 0;
			Game gm;
			gm.set_num(g);
			gm.set_white(atms(0, 1));
			gm.set_black(atms(0, 2));
			int n = -1;
			std::istringstream(atms(0, 3)) >> n;
			if (n < 0) return 0;
			gm.set_moves(n);
			c->game_update(gm);
		} else if (rx.exec(dat, "\\{Game (\\d+): \\S+ vs \\S+ (.*)\\}")) {
			int g = -1;
			std::istringstream(atms(0, 0)) >> g;
			if (g < 0) return 0;
			std::string d = atms(0, 1);
			if (rx.exec(d, "has adjourned") ) c->game_finish(g, "adjourned");
			else if (rx.exec(d, ": (.*)")) c->game_finish(g, atms(0, 0));
			else {
				std::cerr << "strange game result: " << dat << std::endl;
			}
		} else if (rx.exec(dat, "\\{(\\S+) \\[(.*)\\] has conn")) {
			c->player_update(atms(0, 0), atms(0, 1));
		} else if (rx.exec(dat, "\\{(\\S+) has discon")) {
			c->player_remove(atms(0, 0));
		} else if (rx.exec(dat, "\\!\\!(.*)\\!\\!\\: (.*)")) {
			// system message
			// std::cerr << "[system message]" << dat << std::endl;
			c->message_sys(dat);
		} else if (rx.exec(dat, "\\!(.*)\\!\\: (.*)")) {
			// yelling
			c->message_yell(dat);
		} else {
			return 0;
		}
		break;
	case 24: // tell
		if (rx.exec(dat, "\\*(\\S+)\\*:  (.*)")) {
			c->message_tell(dat);
			c->message_tell(atms(0, 0), atms(0, 1));
		} else return 0;
		break;
	case 28: // undo
		if (rx.exec(dat, "Undo in game (\\d+): ")) {
			int g = -1;
			std::istringstream(atms(0, 0)) >> g;
			if (g < 0) return 0;
			c->game_undo(g);
		} else if (rx.exec(dat, "\\S+ undid the")) {
			c->game_undo();
		} else {
			return 0;
		}
		break;
	case 48: //
		if (rx.exec(dat, "Game (\\d+) has been adjourned by (\\S+)")) {
			int g = -1;
			std::istringstream(atms(0, 0)) >> g;
			if (g < 0) return 0;
			std::string ss("adjourned by ");
			ss += atms(0, 1);
			c->game_finish(g, ss);
		} else return 0;
		break;
	case 49:
		if (rx.exec(dat, "Game (\\d+) \\S+ is removing @ (([A-H]|[J-T])\\d+)")) {
// 			std::cerr << "removing @ " << atms(0, 1) << std::endl;
			int g = -1;
			std::istringstream(atms(0, 0)) >> g;
			if (g < 0) return 0;
			c->match_remove(go::Loc(atms(0, 1)), g);
		} else return 0;
		break;
	default:
		return 0;
	}
	// return & parser_misc;
	return (Parser *) this;
}

Misc::Misc(Parser * p) :
	Parser(p)
{
}

Misc::Misc(Control * c) :
	Parser(c)
{
}

Misc::~Misc()
{
}

bool Misc::process()
{
	return false;
}
